@@ -37,13 +37,12 @@
 
 #include <netdb.h>
 
-const size_t   MAX_IPV4_HDR_SIZE       = 60;
-const uint32_t CALIBRATION_CYCLES      = 100,
-               PKT_BURST_PRECISION     = 1000,
-               BUF_SIZE_RESERVE_FACTOR = 10;
+static const size_t   MAX_IPV4_HDR_SIZE       = 60;
+static const uint32_t CALIBRATION_CYCLES      = 100,
+                      PKT_BURST_PRECISION     = 1000,
+                      BUF_SIZE_RESERVE_FACTOR = 10;
 
-int64_t min_rtt, max_rtt, average_rtt;
-char   *prog_name;
+static char *prog_name;
 
 static void get_time(struct timespec *ts)
 {
@@ -56,7 +55,7 @@ static void get_time(struct timespec *ts)
     const clockid_t id = CLOCK_MONOTONIC;
 #else
     const clockid_t id = CLOCK_REALTIME;
-#endif /* CLOCK_XXX */
+#endif
 
     if (clock_gettime(id, ts) < 0) {
         fprintf(stderr, "%s: clock_gettime() failed: %s\n", prog_name, strerror(errno));
@@ -64,7 +63,7 @@ static void get_time(struct timespec *ts)
         ts->tv_sec  = 0;
         ts->tv_nsec = 0;
     }
-#else
+#else /* HAVE_CLOCK_GETTIME */
     struct timeval tv;
 
     if (gettimeofday(&tv, NULL) < 0) {
@@ -79,16 +78,15 @@ static void get_time(struct timespec *ts)
 #endif /* HAVE_CLOCK_GETTIME */
 }
 
-static int64_t ts_sub(struct timespec *ts1, struct timespec *ts2)
+static int64_t ts_sub(const struct timespec *ts1, const struct timespec *ts2)
 {
     return ((int64_t)ts1->tv_sec - (int64_t)ts2->tv_sec) * 1000000 + (ts1->tv_nsec - ts2->tv_nsec) / 1000;
 }
 
-static uint16_t cksum(void *packet, size_t pkt_size)
+static uint16_t cksum(const void *packet, size_t pkt_size)
 {
-    uint16_t buf[IP_MAXPACKET];
+    uint16_t buf[IP_MAXPACKET] = {0};
 
-    memset(buf, 0,      sizeof(buf));
     memcpy(buf, packet, pkt_size);
 
     uint32_t sum = 0;
@@ -131,37 +129,22 @@ static int64_t calibrate_timer(void)
     return sum / CALIBRATION_CYCLES;
 }
 
-static void send_ping4(int sock, struct addrinfo *to_ai, size_t pkt_size, uint16_t ident, bool first_in_burst, uint32_t *transmitted_number, uint64_t *transmitted_volume)
+static void send_ping4(int sock, const struct addrinfo *to_ai, size_t pkt_size, uint16_t ident, bool first_in_burst, uint32_t *transmitted_number, uint64_t *transmitted_volume)
 {
-    char packet[IP_MAXPACKET];
+    char packet[IP_MAXPACKET] = {0};
 
-    struct icmp icmp4;
-
-    memset(&icmp4, 0, sizeof(icmp4));
-
-    icmp4.icmp_type  = ICMP_ECHO;
-    icmp4.icmp_code  = 0;
-    icmp4.icmp_cksum = 0;
-    icmp4.icmp_id    = ident;
-    icmp4.icmp_seq   = htons(*transmitted_number);
+    struct icmp icmp4 = {.icmp_type = ICMP_ECHO, .icmp_code = 0, .icmp_cksum = 0, .icmp_id = ident, .icmp_seq = htons(*transmitted_number)};
 
     memcpy(packet, &icmp4, sizeof(icmp4));
 
-    struct timespec pkt_time;
-
     if (first_in_burst) {
-        struct timespec now;
+        struct timespec pkt_time;
 
-        get_time(&now);
+        get_time(&pkt_time);
 
-        pkt_time.tv_sec  = now.tv_sec;
-        pkt_time.tv_nsec = now.tv_nsec;
-    } else {
-        memset(&pkt_time, 0, sizeof(pkt_time));
+        memcpy(&packet[sizeof(icmp4)], &pkt_time, sizeof(pkt_time));
     }
 
-    memcpy(&packet[sizeof(icmp4)], &pkt_time, sizeof(pkt_time));
-
     icmp4.icmp_cksum = cksum(packet, pkt_size);
 
     memcpy(&packet[offsetof(struct icmp, icmp_cksum)], &icmp4.icmp_cksum, sizeof(icmp4.icmp_cksum));
@@ -178,37 +161,22 @@ static void send_ping4(int sock, struct addrinfo *to_ai, size_t pkt_size, uint16
     (*transmitted_volume) += pkt_size;
 }
 
-static void send_ping6(int sock, struct addrinfo *to_ai, size_t pkt_size, uint16_t ident, bool first_in_burst, uint32_t *transmitted_number, uint64_t *transmitted_volume)
+static void send_ping6(int sock, const struct addrinfo *to_ai, size_t pkt_size, uint16_t ident, bool first_in_burst, uint32_t *transmitted_number, uint64_t *transmitted_volume)
 {
-    char packet[IP_MAXPACKET];
+    char packet[IP_MAXPACKET] = {0};
 
-    struct icmp6_hdr icmp6;
-
-    memset(&icmp6, 0, sizeof(icmp6));
-
-    icmp6.icmp6_type  = ICMP6_ECHO_REQUEST;
-    icmp6.icmp6_code  = 0;
-    icmp6.icmp6_cksum = 0;
-    icmp6.icmp6_id    = ident;
-    icmp6.icmp6_seq   = htons(*transmitted_number);
+    struct icmp6_hdr icmp6 = {.icmp6_type = ICMP6_ECHO_REQUEST, .icmp6_code = 0, .icmp6_cksum = 0, .icmp6_id = ident, .icmp6_seq = htons(*transmitted_number)};
 
     memcpy(packet, &icmp6, sizeof(icmp6));
 
-    struct timespec pkt_time;
-
     if (first_in_burst) {
-        struct timespec now;
+        struct timespec pkt_time;
 
-        get_time(&now);
+        get_time(&pkt_time);
 
-        pkt_time.tv_sec  = now.tv_sec;
-        pkt_time.tv_nsec = now.tv_nsec;
-    } else {
-        memset(&pkt_time, 0, sizeof(pkt_time));
+        memcpy(&packet[sizeof(icmp6)], &pkt_time, sizeof(pkt_time));
     }
 
-    memcpy(&packet[sizeof(icmp6)], &pkt_time, sizeof(pkt_time));
-
     ssize_t res = sendto(sock, packet, pkt_size, 0, to_ai->ai_addr, to_ai->ai_addrlen);
 
     if (res < 0) {
@@ -221,23 +189,12 @@ static void send_ping6(int sock, struct addrinfo *to_ai, size_t pkt_size, uint16
     (*transmitted_volume) += pkt_size;
 }
 
-static bool recv_ping4(int sock, uint16_t ident, uint32_t *received_number, uint64_t *received_volume)
+static bool recv_ping4(int sock, uint16_t ident, uint32_t *received_number, uint64_t *received_volume, int64_t *min_rtt, int64_t *max_rtt, int64_t *average_rtt)
 {
     char packet[IP_MAXPACKET];
 
-    struct iovec iov;
-
-    memset(&iov, 0, sizeof(iov));
-
-    iov.iov_base = packet;
-    iov.iov_len  = sizeof(packet);
-
-    struct msghdr msg;
-
-    memset(&msg, 0, sizeof(msg));
-
-    msg.msg_iov    = &iov;
-    msg.msg_iovlen = 1;
+    struct iovec  iov = {.iov_base = packet, .iov_len    = sizeof(packet)};
+    struct msghdr msg = {.msg_iov  = &iov,   .msg_iovlen = 1};
 
     ssize_t res = recvmsg(sock, &msg, MSG_DONTWAIT);
 
@@ -270,14 +227,14 @@ static bool recv_ping4(int sock, uint16_t ident, uint32_t *received_number, uint
 
                         int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
 
-                        if (min_rtt > rtt) {
-                            min_rtt = rtt;
+                        if (*min_rtt > rtt) {
+                            *min_rtt = rtt;
                         }
-                        if (max_rtt < rtt) {
-                            max_rtt = rtt;
+                        if (*max_rtt < rtt) {
+                            *max_rtt = rtt;
                         }
 
-                        average_rtt = *received_number ? ((average_rtt * (*received_number - 1)) + rtt) / *received_number : average_rtt;
+                        *average_rtt = *received_number ? ((*average_rtt * (*received_number - 1)) + rtt) / *received_number : *average_rtt;
                     }
                 }
             }
@@ -289,23 +246,12 @@ static bool recv_ping4(int sock, uint16_t ident, uint32_t *received_number, uint
     }
 }
 
-static bool recv_ping6(int sock, uint16_t ident, uint32_t *received_number, uint64_t *received_volume)
+static bool recv_ping6(int sock, uint16_t ident, uint32_t *received_number, uint64_t *received_volume, int64_t *min_rtt, int64_t *max_rtt, int64_t *average_rtt)
 {
     char packet[IP_MAXPACKET];
 
-    struct iovec iov;
-
-    memset(&iov, 0, sizeof(iov));
-
-    iov.iov_base = packet;
-    iov.iov_len  = sizeof(packet);
-
-    struct msghdr msg;
-
-    memset(&msg, 0, sizeof(msg));
-
-    msg.msg_iov    = &iov;
-    msg.msg_iovlen = 1;
+    struct iovec  iov = {.iov_base = packet, .iov_len    = sizeof(packet)};
+    struct msghdr msg = {.msg_iov  = &iov,   .msg_iovlen = 1};
 
     ssize_t res = recvmsg(sock, &msg, MSG_DONTWAIT);
 
@@ -331,14 +277,14 @@ static bool recv_ping6(int sock, uint16_t ident, uint32_t *received_number, uint
 
                     int64_t rtt = ts_sub(&now, &pkt_time) / 1000;
 
-                    if (min_rtt > rtt) {
-                        min_rtt = rtt;
+                    if (*min_rtt > rtt) {
+                        *min_rtt = rtt;
                     }
-                    if (max_rtt < rtt) {
-                        max_rtt = rtt;
+                    if (*max_rtt < rtt) {
+                        *max_rtt = rtt;
                     }
 
-                    average_rtt = *received_number ? ((average_rtt * (*received_number - 1)) + rtt) / *received_number : average_rtt;
+                    *average_rtt = *received_number ? ((*average_rtt * (*received_number - 1)) + rtt) / *received_number : *average_rtt;
                 }
             }
         }
@@ -349,14 +295,9 @@ static bool recv_ping6(int sock, uint16_t ident, uint32_t *received_number, uint
     }
 }
 
-static bool resolve_name(bool ipv4_mode, char *name, struct addrinfo **addr_info)
+static bool resolve_name(bool ipv4_mode, const char *name, struct addrinfo **addr_info)
 {
-    struct addrinfo hints;
-
-    memset(&hints, 0, sizeof(hints));
-
-    hints.ai_flags    = AI_CANONNAME;
-    hints.ai_socktype = SOCK_RAW;
+    struct addrinfo hints = {.ai_flags = AI_CANONNAME, .ai_socktype = SOCK_RAW};
 
     if (ipv4_mode) {
         hints.ai_family   = AF_INET;
@@ -377,7 +318,7 @@ static bool resolve_name(bool ipv4_mode, char *name, struct addrinfo **addr_info
     }
 }
 
-int main(int argc, char **argv)
+int main(int argc, char *argv[])
 {
     prog_name = basename(argv[0]);
 
@@ -551,13 +492,12 @@ int main(int argc, char **argv)
                 printf("Target: %s (%s), transfer speed: %" PRIu32 " kbps, packet size: %zu bytes, traffic volume: %" PRIu64 " bytes\n",
                        target, addr_buf, kbps, pkt_size, volume);
 
-                min_rtt     = INT64_MAX;
-                max_rtt     = 0;
-                average_rtt = 0;
-
                 bool     finish             = false;
                 uint32_t transmitted_number = 0,
                          received_number    = 0;
+                int64_t  min_rtt            = INT64_MAX,
+                         max_rtt            = 0,
+                         average_rtt        = 0;
                 uint64_t transmitted_volume = 0,
                          received_volume    = 0;
 
@@ -641,8 +581,8 @@ int main(int argc, char **argv)
                         int n = select(sock + 1, &fds, NULL, NULL, &timeout);
 
                         if (n > 0) {
-                            while (ipv4_mode ? recv_ping4(sock, ident, &received_number, &received_volume) :
-                                               recv_ping6(sock, ident, &received_number, &received_volume)) {
+                            while (ipv4_mode ? recv_ping4(sock, ident, &received_number, &received_volume, &min_rtt, &max_rtt, &average_rtt) :
+                                               recv_ping6(sock, ident, &received_number, &received_volume, &min_rtt, &max_rtt, &average_rtt)) {
                                 if (received_number >= transmitted_number) {
                                     break;
                                 }
